from tempfile import mktemp
from unittest import TestCase
from django.test import TestCase as DjangoTestCase
from django.core import management
from django.contrib.auth import models as auth_models

from jnt_admin_tools.dashboard import AppIndexDashboard
from jnt_admin_tools.dashboard.modules import DashboardModule, Group


class ManagementCommandTest(DjangoTestCase):
    def test_customdashboard(self):
        # check that customdashboard command doesn't raise exceptions
        file_name = mktemp()
        management.call_command("customdashboard", file=file_name)
        # and fails if file is already here
        try:
            management.call_command("customdashboard", file=file_name)
            assert False
        except:
            pass


class AppIndexDashboardTest(TestCase):
    def test_models(self):
        models = [
            "django.contrib.auth.models.User",
            "django.contrib.auth.models.Group",
        ]
        board = AppIndexDashboard("Auth", models)
        self.assertEqual(
            board.get_app_model_classes(),
            [auth_models.User, auth_models.Group],
        )


__test__ = {
    "DashboardModule.is_empty": DashboardModule.is_empty,
    "DashboardModule.render_css_classes": DashboardModule.render_css_classes,
    "Group.is_empty": Group.is_empty,
}



from django.conf import settings
from django.conf.urls import include
from django.urls import re_path

urlpatterns = []

if "jnt_admin_tools.menu" in settings.INSTALLED_APPS:
    urlpatterns.append(
        re_path(r"^menu/", include("jnt_admin_tools.menu.urls"))
    )

if "jnt_admin_tools.dashboard" in settings.INSTALLED_APPS:
    urlpatterns.append(
        re_path(r"^dashboard/", include("jnt_admin_tools.dashboard.urls")),
    )



"""
Menu template tags, the following menu tags are available:

 * ``{% admin_tools_render_menu %}``
 * ``{% admin_tools_render_menu_item %}``
 * ``{% admin_tools_render_menu_css %}``

To load the menu tags in your templates: ``{% load admin_tools_menu_tags %}``.
"""

from jnt_admin_tools.menu import items
from jnt_admin_tools.menu.models import Bookmark
from jnt_admin_tools.menu.utils import get_admin_menu
from jnt_admin_tools.utils import get_admin_site_name
from django import template
from django.urls import reverse

register = template.Library()
tag_func = register.inclusion_tag(
    "jnt_admin_tools/menu/dummy.html", takes_context=True
)


def admin_tools_render_menu(context, menu=None):
    """
    Template tag that renders the menu, it takes an optional ``Menu`` instance
    as unique argument, if not given, the menu will be retrieved with the
    ``get_admin_menu`` function.
    """
    if menu is None:
        menu = get_admin_menu(context)

    menu.init_with_context(context)
    has_bookmark_item = False
    bookmark = None
    if len([c for c in menu.children if isinstance(c, items.Bookmarks)]) > 0:
        has_bookmark_item = True
        url = context["request"].get_full_path()
        try:
            bookmark = Bookmark.objects.filter(
                user=context["request"].user, url=url
            )[0]
        except:
            pass

    context.update(
        {
            "template": menu.template,
            "menu": menu,
            "has_bookmark_item": has_bookmark_item,
            "bookmark": bookmark,
            "admin_url": reverse("%s:index" % get_admin_site_name(context)),
        }
    )
    return context


admin_tools_render_menu = tag_func(admin_tools_render_menu)


def admin_tools_render_menu_item(context, item, index=None):
    """
    Template tag that renders a given menu item, it takes a ``MenuItem``
    instance as unique parameter.
    """
    item.init_with_context(context)

    context.update(
        {
            "template": item.template,
            "item": item,
            "index": index,
            "selected": item.is_selected(context["request"]),
            "admin_url": reverse("%s:index" % get_admin_site_name(context)),
        }
    )
    return context


admin_tools_render_menu_item = tag_func(admin_tools_render_menu_item)


def admin_tools_render_menu_css(context, menu=None):
    """
    Template tag that renders the menu css files,, it takes an optional
    ``Menu`` instance as unique argument, if not given, the menu will be
    retrieved with the ``get_admin_menu`` function.
    """
    if menu is None:
        menu = get_admin_menu(context)

    context.update(
        {
            "template": "jnt_admin_tools/menu/css.html",
            "css_files": menu.Media.css,
        }
    )
    return context


admin_tools_render_menu_css = tag_func(admin_tools_render_menu_css)



"""
Dashboard utilities.
"""
from importlib import import_module

from django.conf import settings
from django.urls import reverse
from django.utils.text import capfirst
from jnt_admin_tools.dashboard.registry import Registry
from jnt_admin_tools.utils import get_admin_site


def get_dashboard(context, location):
    """
    Returns the dashboard that match the given ``location``
    (index or app_index).
    """
    if location == "index":
        return get_index_dashboard(context)
    elif location == "app_index":
        return get_app_index_dashboard(context)
    raise ValueError('Invalid dashboard location: "%s"' % location)


def _get_dashboard_cls(dashboard_cls, context):
    if isinstance(dashboard_cls, dict):
        curr_url = context.get("request").path
        for key in dashboard_cls:
            admin_site_mod, admin_site_inst = key.rsplit(".", 1)
            admin_site_mod = import_module(admin_site_mod)
            admin_site = getattr(admin_site_mod, admin_site_inst)
            admin_url = reverse("%s:index" % admin_site.name)
            if curr_url.startswith(admin_url):
                mod, inst = dashboard_cls[key].rsplit(".", 1)
                mod = import_module(mod)
                return getattr(mod, inst)
    else:
        mod, inst = dashboard_cls.rsplit(".", 1)
        mod = import_module(mod)
        return getattr(mod, inst)
    raise ValueError('Dashboard matching "%s" not found' % dashboard_cls)


def get_index_dashboard(context):
    """
    Returns the admin dashboard defined by the user or the default one.
    """
    return _get_dashboard_cls(
        getattr(
            settings,
            "ADMIN_TOOLS_INDEX_DASHBOARD",
            "jnt_admin_tools.dashboard.dashboards.DefaultIndexDashboard",
        ),
        context,
    )()


def get_app_index_dashboard(context):
    """
    Returns the admin dashboard defined by the user or the default one.
    """
    # this is a mess, needs cleanup !
    app = context["app_list"][0]
    model_list = []
    app_label = None
    app_title = app["name"]
    admin_site = get_admin_site(context=context)

    for model, model_admin in admin_site._registry.items():
        if app["app_label"] == model._meta.app_label:
            split = model.__module__.find(model._meta.app_label)
            app_label = model.__module__[0:split] + model._meta.app_label
            for m in app["models"]:
                if m["name"] == capfirst(model._meta.verbose_name_plural):
                    mod = "%s.%s" % (model.__module__, model.__name__)
                    model_list.append(mod)

    # if an app has registered its own dashboard, use it
    if app_label is not None and app_label in Registry.registry:
        return Registry.registry[app_label](app_title, model_list)

    # try to discover a general app_index dashboard (with fallback to the
    # default dashboard)
    return _get_dashboard_cls(
        getattr(
            settings,
            "ADMIN_TOOLS_APP_INDEX_DASHBOARD",
            "jnt_admin_tools.dashboard.dashboards.DefaultAppIndexDashboard",
        ),
        context,
    )(app_title, model_list)



import os
from django.core.management.base import BaseCommand, CommandError
from django.template.loader import render_to_string


DEFAULT_FILE = "menu.py"


class Command(BaseCommand):
    help = (
        "Creates a template file containing the base code to get you "
        "started with your custom menu"
    )
    args = ["file"]

    def handle(self, file=None, **options):
        project_name = os.path.basename(os.getcwd())
        dst = file is not None and file or DEFAULT_FILE
        if os.path.exists(dst):
            raise CommandError('Error: file "%s" already exists' % dst)
        open(dst, "w").write(
            render_to_string(
                "jnt_admin_tools/menu/menu.txt",
                {
                    "project": project_name,
                    "file": os.path.basename(dst).split(".")[0],
                },
            )
        )
        self.stdout.write('"%s" written.' % os.path.join(dst))



from .autocomplete_fields import AdminAutocompleteFieldsMixin
from .generic_foreign_key import GenericForeignKeyAdminMixin
from .generic_foreign_key_inline import GenericForeignKeyInlineAdminMixin
from .admin_clickable_links import AdminClickableLinksMixin



from jnt_admin_tools.mixins.base_generic_foreign_key import (
    BaseGenericForeignKeyMixin,
)


class GenericForeignKeyInlineAdminMixin(BaseGenericForeignKeyMixin):
    def get_formset(self, request, obj, **kwargs):
        formset = super().get_formset(request, obj, **kwargs)
        self._set_declared_gfk_to_form(formset.form)
        self._set_validators_gfk(formset.form)

        return formset



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['jnt_admin_tools',
 'jnt_admin_tools.dashboard',
 'jnt_admin_tools.dashboard.management',
 'jnt_admin_tools.dashboard.management.commands',
 'jnt_admin_tools.dashboard.migrations',
 'jnt_admin_tools.dashboard.templatetags',
 'jnt_admin_tools.db.fields',
 'jnt_admin_tools.fields',
 'jnt_admin_tools.menu',
 'jnt_admin_tools.menu.management',
 'jnt_admin_tools.menu.management.commands',
 'jnt_admin_tools.menu.migrations',
 'jnt_admin_tools.menu.templatetags',
 'jnt_admin_tools.mixins',
 'jnt_admin_tools.services',
 'jnt_admin_tools.static.jnt_admin_tools.css',
 'jnt_admin_tools.static.jnt_admin_tools.css.widgets',
 'jnt_admin_tools.static.jnt_admin_tools.js.widgets',
 'jnt_admin_tools.templates',
 'jnt_admin_tools.templates.jnt_admin_tools',
 'jnt_admin_tools.templatetags',
 'jnt_admin_tools.theming',
 'jnt_admin_tools.theming.templatetags',
 'jnt_admin_tools.widgets']

package_data = \
{'': ['*'],
 'jnt_admin_tools': ['locale/ar/LC_MESSAGES/*',
                     'locale/bg/LC_MESSAGES/*',
                     'locale/bn/LC_MESSAGES/*',
                     'locale/ca/LC_MESSAGES/*',
                     'locale/cs/LC_MESSAGES/*',
                     'locale/da/LC_MESSAGES/*',
                     'locale/de/LC_MESSAGES/*',
                     'locale/el/LC_MESSAGES/*',
                     'locale/en/LC_MESSAGES/*',
                     'locale/es/LC_MESSAGES/*',
                     'locale/es_AR/LC_MESSAGES/*',
                     'locale/fi/LC_MESSAGES/*',
                     'locale/fr/LC_MESSAGES/*',
                     'locale/he/LC_MESSAGES/*',
                     'locale/hu/LC_MESSAGES/*',
                     'locale/it/LC_MESSAGES/*',
                     'locale/ja/LC_MESSAGES/*',
                     'locale/nl/LC_MESSAGES/*',
                     'locale/pl/LC_MESSAGES/*',
                     'locale/pt/LC_MESSAGES/*',
                     'locale/pt_BR/LC_MESSAGES/*',
                     'locale/ru/LC_MESSAGES/*',
                     'locale/sk/LC_MESSAGES/*',
                     'locale/sv/LC_MESSAGES/*',
                     'locale/tr/LC_MESSAGES/*',
                     'locale/uk/LC_MESSAGES/*',
                     'locale/zh_CN/LC_MESSAGES/*',
                     'locale/zh_TW/LC_MESSAGES/*',
                     'static/jnt_admin_tools/images/*',
                     'static/jnt_admin_tools/js/*',
                     'static/jnt_admin_tools/js/autocomplete_filter/*'],
 'jnt_admin_tools.dashboard': ['static/jnt_admin_tools/css/*',
                               'static/jnt_admin_tools/css/jquery/*',
                               'static/jnt_admin_tools/css/jquery/images/*',
                               'static/jnt_admin_tools/js/*',
                               'static/jnt_admin_tools/js/jquery/*',
                               'templates/admin/*',
                               'templates/jnt_admin_tools/dashboard/*',
                               'templates/jnt_admin_tools/dashboard/modules/*'],
 'jnt_admin_tools.menu': ['static/jnt_admin_tools/css/*',
                          'static/jnt_admin_tools/js/*',
                          'templates/admin/*',
                          'templates/jnt_admin_tools/menu/*'],
 'jnt_admin_tools.static.jnt_admin_tools.css': ['autocomplete_filter/*'],
 'jnt_admin_tools.templates.jnt_admin_tools': ['autocomplete_filter/*',
                                               'widgets/*'],
 'jnt_admin_tools.theming': ['static/jnt_admin_tools/css/*',
                             'static/jnt_admin_tools/images/*',
                             'templates/admin/*']}

install_requires = \
['django>=3']

setup_kwargs = {
    'name': 'jnt-django-admin-tools',
    'version': '0.9.19',
    'description': 'A collection of tools for the django administration interface',
    'long_description': None,
    'author': 'Junte',
    'author_email': 'tech@junte.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""
This module contains the base classes for menu and menu items.
"""
# for backward-compatibility
from jnt_admin_tools import menu
from jnt_admin_tools.deprecate_utils import import_path_is_changed
from jnt_admin_tools.menu import items
from django.conf import settings
from django.db import models

user_model = getattr(settings, "AUTH_USER_MODEL", "auth.User")


class Bookmark(models.Model):
    """
    This model represents a user created bookmark.
    """

    user = models.ForeignKey(user_model, on_delete=models.CASCADE)
    url = models.CharField(max_length=255)
    title = models.CharField(max_length=255)

    def __str__(self):
        return f"{self.title} - {self.url}"

    class Meta:
        db_table = "admin_tools_menu_bookmark"
        ordering = ("id",)


class Menu(
    import_path_is_changed(
        "jnt_admin_tools.menu.models.Menu", "jnt_admin_tools.menu.Menu"
    ),
    menu.Menu,
):
    pass


class DefaultMenu(
    import_path_is_changed(
        "jnt_admin_tools.menu.models.DefaultMenu",
        "jnt_admin_tools.menu.DefaultMenu",
    ),
    menu.DefaultMenu,
):
    pass


class MenuItem(
    import_path_is_changed(
        "jnt_admin_tools.menu.models.MenuItem",
        "jnt_admin_tools.menu.items.MenuItem",
    ),
    items.MenuItem,
):
    pass


class AppListMenuItem(
    import_path_is_changed(
        "jnt_admin_tools.menu.models.AppListMenuItem",
        "jnt_admin_tools.menu.items.AppList",
    ),
    items.AppList,
):
    pass


class BookmarkMenuItem(
    import_path_is_changed(
        "jnt_admin_tools.menu.models.BookmarkMenuItem",
        "jnt_admin_tools.menu.items.Bookmarks",
    ),
    items.Bookmarks,
):
    pass



from django.forms.models import ModelChoiceIterator
from jnt_admin_tools.widgets import PermissionSelectMultipleWidget


class PermissionSelectMultipleReadonlyWidget(PermissionSelectMultipleWidget):
    """Permissions multiple readonly widget. """

    def __init__(self, *args, **kwargs):
        """Initializing."""
        self._formfield = kwargs.pop("formfield")
        super().__init__(*args, **kwargs)

        self.extra_data["is_readonly"] = True
        self.choices = ModelChoiceIterator(self._formfield)

    def render(self, name, value, *args, **kwargs):
        """Render html-string."""
        self.groups_permissions = value.all()
        return super().render(name=name, value=value.all(), *args, **kwargs)
